package cn.com.qmth.am.task;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.annotation.PostConstruct;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import com.qmth.boot.core.concurrent.service.ConcurrentService;

import cn.com.qmth.am.config.SysProperty;
import cn.com.qmth.am.consumer.OcrConsumer;
import cn.com.qmth.am.entity.QuestionEntity;
import cn.com.qmth.am.entity.StudentScoreEntity;
import cn.com.qmth.am.enums.LockType;
import cn.com.qmth.am.service.QuestionService;
import cn.com.qmth.am.service.StudentScoreService;
import cn.com.qmth.am.utils.SpringContextHolder;

@Service
public class OcrJob {

    @Autowired
    private StudentScoreService studentScoreService;

    @Autowired
    private QuestionService questionService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private SysProperty sysProperty;

    private ExecutorService executor;

    @PostConstruct
    public void initExecutor() {
        int threadCount = sysProperty.getThreadCount();
        executor = new ThreadPoolExecutor(threadCount, threadCount, 0L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(threadCount * 2), r -> {
                    Thread t = new Thread(r);
                    return t;
                }, (r, executor) -> {
                    if (!executor.isShutdown()) {
                        try {
                            executor.getQueue().put(r);
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                    }
                });
    }

    @Scheduled(fixedDelay = 5 * 1000, initialDelay = 5 * 1000)
    public void doJob() {
        if (!sysProperty.getOcrTaskEnable()) {
            return;
        }

        List<QuestionEntity> qs = questionService.list();
        if (CollectionUtils.isEmpty(qs)) {
            return;
        }
        List<StudentScoreEntity> scores = studentScoreService.findAllToOcr();
        if (CollectionUtils.isEmpty(scores)) {
            return;
        }
        boolean lock = concurrentService.getReadWriteLock(LockType.OCR.name()).writeLock().tryLock();
        try {
            if (!lock) {
                return;
            }
            this.dispose(scores, qs);
        } finally {
            if (lock) {
                concurrentService.getReadWriteLock(LockType.OCR.name()).writeLock().unlock();
            }
        }
    }

    private void dispose(List<StudentScoreEntity> scores, List<QuestionEntity> qs) {
        Map<String, Map<Long, QuestionEntity>> qmap = new HashMap<>();
        for (QuestionEntity q : qs) {
            String key = q.getExamId() + "-" + q.getSubjectCode();
            Map<Long, QuestionEntity> tem = qmap.get(key);
            if (tem == null) {
                tem = new HashMap<>();
                qmap.put(key, tem);
            }
            tem.put(q.getId(), q);
        }
        CountDownLatch endGate = new CountDownLatch(scores.size());
        for (StudentScoreEntity score : scores) {
            if (!sysProperty.getOcrTaskEnable()) {
                return;
            }
            String key = score.getExamId() + "-" + score.getSubjectCode();
            Map<Long, QuestionEntity> tem = qmap.get(key);
            if (tem != null) {
                OcrConsumer com = SpringContextHolder.getBean(OcrConsumer.class);
                com.setScore(score);
                com.setQuetions(tem);
                com.setEndGate(endGate);
                executor.execute(com);
            }
        }
        try {
            endGate.await();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

}
